unit OpenDirDialog;

interface

uses
  Windows, Messages, SysUtils, Classes, Controls, Graphics, shlobj, Activex;

type
    TOnFolderChanged=procedure(Sender:TObject; Path:String) of Object;

type
    TOpenDirOptions=set of(odShowDir,odShowComputer,odShowPrinter,odShowFiles);
    TOpenDirOptionsEx=set of(odxShowExtendDlg);

type
  TOpenDirDialog = class(TComponent)
  private
    { Private declarations }
    FBrowseInfo:BROWSEINFO;
    FItemIdList:PITEMIDLIST;
    FOwner:HWND;
    FHandle:HWND;
    FPath:String;
    FTitle:String;
    FOptions:TOpenDirOptions;
    FOptionsEx:TOpenDirOptionsEx;
    FOnCreate:TNotifyEvent;
    FOnChange:TOnFolderChanged;
    procedure SetOptions(Value:TOpenDirOptions);
    procedure SetOptionsEx(Value:TOpenDirOptionsEx);
  protected
    { Protected declarations }
  public
    { Public declarations }
    constructor Create(AOwner:TComponent); override;
    destructor Destroy; override;
    function Execute:Boolean; virtual;
  published
    { Published declarations }
    property Title:String read FTitle write FTitle;
    property Path:String read FPath write FPath;
    property Options:TOpenDirOptions read FOptions write SetOptions;
    property OptionsEx:TOpenDirOptionsEx read FOptionsEx write SetOptionsEx;
    property OnCreate:TNotifyEvent read FOnCreate write FOnCreate;
    property OnChange:TOnFolderChanged read FOnChange write FOnChange;
    property Handle:HWND read FHandle;
  end;

  function BrowseFolderCallBackProc(hWnd:HWND; uMsg:Integer; LParam:Integer; WParam:Integer):integer; stdcall;

procedure Register;

implementation

/////{$R OpenDirDialog.dcr}

var OpenDirDlg:TOpenDirDialog;

procedure Register;
begin
  RegisterComponents('Dialogs', [TOpenDirDialog]);
end;


function BrowseFolderCallBackProc(hWnd:HWND; uMsg:Integer; LParam:Integer; WParam:Integer):Integer;
var tmp:PChar;
    Path:String;
begin
     case uMsg of
        BFFM_INITIALIZED:
                begin
                     OpenDirDlg.FHandle := hWnd;
                     SendMessage(hWnd,BFFM_SETSELECTION,1,Integer(PChar(OpenDirDlg.FPath)));
                     if Assigned(OpenDirDlg.FOnCreate) then OpenDirDlg.FOnCreate(OpenDirDlg);
                end;
        BFFM_SELCHANGED:
                begin
                     if Assigned(OpenDirDlg.FOnChange) then begin
                        GetMem(tmp,MAX_PATH);
                        SHGetPathFromIDList(PITEMIDLIST(lParam),tmp);
                        Path := StrPas(tmp);
                        FreeMem(tmp,MAX_PATH);
                        OpenDirDlg.FOnChange(OpenDirDlg,Path);
                     end;
                end;
        BFFM_VALIDATEFAILED:
                begin
                end;
     end;
     Result := 0;
end;


constructor TOpenDirDialog.Create(AOwner:TComponent);
begin
     inherited Create(AOwner);
     FOwner := TWinControl(AOwner).Handle;
     FBrowseInfo.ulFlags := 0;
     FItemIdList := nil;
     OpenDirDlg := Self;
     FOptionsEx := [];
end;


destructor TOpenDirDialog.Destroy;
begin
     inherited Destroy;
end;


function TOpenDirDialog.Execute:Boolean;
var tmp:PChar;
begin
     CoInitialize(nil);
     GetMem(tmp,MAX_PATH);
     with FBrowseInfo do begin
          hwndOwner := FOwner;
          lpszTitle := PChar(FTitle);
          pszDisplayName := tmp;
          pidlRoot := nil;
          lpfn := @BrowseFolderCallBackProc;
     end;

     FItemIdList := SHBrowseForFolder(FBrowseInfo);
     if FItemIdList<>nil then begin
        SHGetPathFromIDList(FItemIdList,tmp);
        FPath := StrPas(tmp);
        Result := True;
     end
     else Result := False;

     FreeMem(tmp,MAX_PATH);
     CoTaskMemFree(FItemIdList);
end;

procedure TOpenDirDialog.SetOptions(Value:TOpenDirOptions);
begin
     FOptions := Value;
     if odShowDir in FOptions then
        FBrowseInfo.ulFlags := FBrowseInfo.ulFlags or BIF_RETURNONLYFSDIRS;
     if odShowComputer in FOptions then
        FBrowseInfo.ulFlags := FBrowseInfo.ulFlags or BIF_BROWSEFORCOMPUTER;
     if odShowPrinter in FOptions then
        FBrowseInfo.ulFlags := FBrowseInfo.ulFlags or BIF_BROWSEFORPRINTER;
     if odShowFiles in  FOptions then
        FBrowseInfo.ulFlags := FBrowseInfo.ulFlags or BIF_BROWSEINCLUDEFILES;
end;

procedure TOpenDirDialog.SetOptionsEx(Value:TOpenDirOptionsEx);
begin
     if Value <> FOptionsEx then begin
        FOptionsEx := Value;
        FBrowseInfo.ulFlags := FBrowseInfo.ulFlags or $50;
     end;
end;

end.
